package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesJisShippingService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesSortShippingCheckService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesSortShippingCheckModel;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesJisShipping;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesLoadingList;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagement;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagementDetail;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @Description :
 * @Reference :
 * @Author :
 * @CreateDate : 2024-04-26 15:38
 * @Modify:
 **/
@Service
@Slf4j
public class MesSortShippingCheckService implements IMesSortShippingCheckService {

    @Autowired
    private MesShippingOrderManagementRepository shippingOrderManagementRepository;
    @Autowired
    private MesShippingOrderManagementDetailRepository shippingOrderManagementDetailRepository;
    @Autowired
    private MesProduceSnRepository produceSnRepository;

    @Autowired
    private MesPartShippingGroupRepository partShippingGroupRepository;
    @Autowired
    private MesLoadingListRepository ListRepository;

    @Autowired
    private MesLoadingListDetailRepository listDetailRepository;

    @Autowired
    private MesJisShippingRepository jisShippingRepository;

    @Autowired
    private IMesJisShippingService jisShippingService;

    @Override
    public MesSortShippingCheckModel queryShippingOrderNo(MesShippingOrderManagement shippingOrderManagement) {


        //校验发运单是否存在
        DdlPackBean orderManagementPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.organizeCode);
        DdlPreparedPack.getStringEqualPack(shippingOrderManagement.getShippingCode(), "shippingCode", orderManagementPackBean);
        MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getByProperty(orderManagementPackBean);
        if (StringUtils.isEmpty(orderManagement)) {
            throw new ImppBusiException(String.format("【%s】此发运单不存在，请检查数据！", shippingOrderManagement.getShippingCode()));
        }

        //校验发运单状态 必须是未装车条码
        if (orderManagement.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue() && orderManagement.getStatus() != MesExtEnumUtil.SHIPPING_ORDER_STATUS.PUBLISHED.getValue()) {
            throw new ImppBusiException(String.format("【%s】此发运单状态为【%s】,请选择状态为【%s】状态的发运单", shippingOrderManagement.getShippingCode(), orderManagement.getStatus(),
                    MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue()));
        }


        //根据扫描的发运单找到是否装车配置，若是，则根据发运单查询是否生成装车单，若没生成装车单则提示此发运单配置是否装车为是,但没生成装车单，请检查数据！
        if (orderManagement.getIsLoading() == MesCommonConstant.TRUE_INTEGER) {

            if (StringUtils.isEmpty(orderManagement.getOrderCode())) {
                throw new ImppBusiException(String.format("【%s】此发运单配置是否装车为是,但没绑定装车单号，请检查数据！", shippingOrderManagement.getShippingCode()));
            }

            DdlPackBean orderDetailPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.organizeCode);
            DdlPreparedPack.getStringEqualPack(orderManagement.getOrderCode(), "orderCode", orderDetailPackBean);
            MesLoadingList loadingList = ListRepository.getByProperty(orderDetailPackBean);
            if (StringUtils.isEmpty(loadingList)) {
                throw new ImppBusiException(String.format("【%s】此发运单配置是否装车为是,但没生成装车单，请检查数据！", shippingOrderManagement.getShippingCode()));
            }

            //判断是否先装车
            if (orderManagement.getIsFirstInstall() == MesCommonConstant.TRUE_INTEGER) {

                //若没进行装车校验，则提示【%s】此发运单对应的【%s】此装车单状态为【%s】，请选择状态为【%s】的数据
                if (loadingList.getStatus() != MesExtEnumUtil.LOADING_DETAIL_ORDER_STATUS.LOADED.getValue()) {
                    throw new ImppBusiException(String.format("【%s】此发运单对应的【%s】此装车单状态为【%s】，请选择状态为【%s】的数据！", shippingOrderManagement.getShippingCode(),
                            loadingList.getOrderCode(), MesExtEnumUtil.LOADING_ORDER_STATUS.valueOfDescription(loadingList.getStatus()), MesExtEnumUtil.LOADING_ORDER_STATUS.LOADED.getDescription()));
                }
            }
        }

        MesSortShippingCheckModel model = new MesSortShippingCheckModel();
        if (!StringUtils.isEmpty(orderManagement.getCheckSeqCode())) {
            model.setIsScanLocationCode(true);
        }


        DdlPackBean orderDetailPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.organizeCode);
        DdlPreparedPack.getNumEqualPack(orderManagement.getId(), "pid", orderDetailPackBean);
        List<MesShippingOrderManagementDetail> detailList = shippingOrderManagementDetailRepository.findByHqlWhere(orderDetailPackBean);


        //查询最近扫描完成的装车单号
        DdlPackBean recentlyOrderPackBean = DdlPackBean.getDdlPackBean(shippingOrderManagement.organizeCode);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue(), "status", recentlyOrderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"modifyDatetime"}, recentlyOrderPackBean);
        MesShippingOrderManagement recentlyVehiclesOrder = shippingOrderManagementRepository.getByProperty(recentlyOrderPackBean);

        model.setDetailList(detailList);
        model.setShippingCode(orderManagement.getShippingCode());
        if (!StringUtils.isEmpty(recentlyVehiclesOrder)) {

            model.setRecentlyScanShippingCode(recentlyVehiclesOrder.getShippingCode());
        }

        return model;
    }

    @Override
    public MesSortShippingCheckModel saveSn(MesSortShippingCheckModel model, String org) {

        //校验条码
        MesProduceSn produceSn = checkMesProduceSn(model, org);

        //校验发运单是否存在
        MesShippingOrderManagement orderManagement = checkMesShippingOrderManagement(model, org);

        //根据条码找到对应的物料号，若物料号一致，则修改扫描flg，将flg改为true.
        boolean scanFlg = false;
        for (MesShippingOrderManagementDetail detail : model.getDetailList()) {
            if (detail.getPartNo().equals(produceSn.getPartNo()) && StringUtils.isEmpty(detail.getBarcode())) {
                scanFlg = true;
                detail.setBarcode(produceSn.getSerialNumber());
                if (StringUtils.isEmpty(orderManagement.getCheckSeqCode())) {
                    detail.setIsScanFlg(MesCommonConstant.TRUE_INTEGER);

                    //根据是否启动记忆判断 若启用记忆，则根据保存数据库
                    if (orderManagement.getIsEnableMemory() == MesCommonConstant.TRUE_INTEGER) {

                        orderManagement.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue());
                        ConvertBean.serviceModelUpdate(orderManagement, AuthUtil.getSessionUser().getUserName());
                        shippingOrderManagementRepository.save(orderManagement);

                        detail.setStatus(MesExtEnumUtil.SHIPPING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue());
                        ConvertBean.serviceModelUpdate(detail, AuthUtil.getSessionUser().getUserName());
                        shippingOrderManagementDetailRepository.save(detail);

                        produceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.SHIPPED.getValue());
                        ConvertBean.serviceModelUpdate(detail, AuthUtil.getSessionUser().getUserName());
                        produceSnRepository.save(produceSn);
                    }
                }
                break;
            }
        }

        if (!scanFlg) {
            throw new ImppBusiException(String.format("【%s】此条码对应的【%s】物料号与发运单中物料号不匹配，请检查数据！", model.getSn(), produceSn.getPartNo()));
        }

        Long count = model.getDetailList().stream().filter(k -> k.getIsScanFlg() == MesCommonConstant.TRUE_INTEGER).count();
        model.setIsScanCount(count.intValue());

        //当前发运单全部扫描完成 并且不需要扫位置码
        if (count.intValue() == model.getDetailList().size() && StringUtils.isEmpty(orderManagement.getCheckSeqCode())) {

            //保存数据库
            saveDate(orderManagement, model, org);
        }

        return model;
    }

    @Override
    public MesSortShippingCheckModel saveSnAndLocationCode(MesSortShippingCheckModel model, String org) {

        //校验条码
        MesProduceSn produceSn = checkMesProduceSn(model, org);


        //校验发运单是否存在
        MesShippingOrderManagement orderManagement = checkMesShippingOrderManagement(model, org);

        //根据条码找到对应的物料号，若物料号一致，则修改扫描flg，将flg改为true.
        boolean scanFlg = false;
        for (MesShippingOrderManagementDetail detail : model.getDetailList()) {
            if (detail.getPartNo().equals(produceSn.getPartNo())) {
                scanFlg = true;
                detail.setIsScanFlg(MesCommonConstant.TRUE_INTEGER);
                detail.setBarcode(produceSn.getSerialNumber());
                break;
            }
        }

        if (!scanFlg) {
            throw new ImppBusiException(String.format("【%s】此条码对应的【%s】物料号与发运单中物料号不匹配，请检查数据！", model.getSn(), produceSn.getPartNo()));
        }

        Long count = model.getDetailList().stream().filter(k -> k.getIsScanFlg() == MesCommonConstant.TRUE_INTEGER).count();

        //当前发运单全部扫描完成
        if (count.intValue() == model.getDetailList().size()) {

            //保存数据库
            saveDate(orderManagement, model, org);

        }

        return model;
    }

    @Override
    public MesShippingOrderManagement queryShippingOrderNoByAutoPushOrder(String shippingGroupNo, String org) {

        //查询最近扫描完成的装车单号
        DdlPackBean orderPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.SHIPPING_ORDER_STATUS.CREATE.getValue(), "status", orderPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"seq"}, orderPackBean);
        MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getByProperty(orderPackBean);

        return orderManagement;

    }


    private MesProduceSn checkMesProduceSn(MesSortShippingCheckModel model, String org) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(model.getSn(), "productSn", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        MesProduceSn produceSn = produceSnRepository.getByProperty(packBean);


        if (StringUtils.isEmpty(produceSn)) {
            throw new ImppBusiException(String.format("【%s】此产品条码不存在，请检查数据！", model.getSn()));
        }
        //校验产品条码状态 必须是状态为已下线条码
        if (produceSn.getSnStatus() != MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getValue()) {
            throw new ImppBusiException(String.format("【%s】此条码状态为【%s】,请选择状态为【%s】状态的条码", model.getSn(), MesExtEnumUtil.PRODUCE_SN_STATUS.valueOfDescription(produceSn.getSnStatus()),
                    MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getDescription()));
        }
        //校验产品质量状态 必须是状态为合格条码
        if (produceSn.getQcStatus() != MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue()) {
            throw new ImppBusiException(String.format("【%s】此条码质量状态为【%s】,请选择【%s】状态的条码", model.getSn(), MesExtEnumUtil.PRODUCE_QC_STATUS.valueOfDescription(produceSn.getQcStatus()),
                    MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getDescription()));
        }

        Optional optional = model.getDetailList().stream().filter(k -> !StringUtils.isEmpty(k.getBarcode()) && k.getBarcode().equals(model.getSn())).findFirst();
        if (optional.isPresent()) {
            throw new ImppBusiException(String.format("【%s】此条码已经扫描过，请检查数据！", model.getSn()));
        }
        return produceSn;
    }

    private MesShippingOrderManagement checkMesShippingOrderManagement(MesSortShippingCheckModel model, String org) {
        //校验发运单是否存在
        DdlPackBean orderManagementPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(model.getShippingCode(), "shippingCode", orderManagementPackBean);
        MesShippingOrderManagement orderManagement = shippingOrderManagementRepository.getByProperty(orderManagementPackBean);
        if (StringUtils.isEmpty(orderManagement)) {
            throw new ImppBusiException(String.format("【%s】此发运单不存在，请检查数据！", model.getShippingCode()));
        }
        return orderManagement;
    }

    public void saveDate(MesShippingOrderManagement orderManagement, MesSortShippingCheckModel model, String org) {


        //发运单状态修改为校验完成
        orderManagement.setStatus(MesExtEnumUtil.SHIPPING_ORDER_STATUS.SHIPPED.getValue());

        //发运单明细状态修改为校验完成
        DdlPackBean orderManagementDetailPackBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getNumEqualPack(orderManagement.getId(), "pid", orderManagementDetailPackBean);
        List<MesShippingOrderManagementDetail> orderManagementDetailList = shippingOrderManagementDetailRepository.findByHqlWhere(orderManagementDetailPackBean);

        List<MesProduceSn> produceSnList = new ArrayList<>();
        for (MesShippingOrderManagementDetail detail : orderManagementDetailList) {
            detail.setStatus(MesExtEnumUtil.SHIPPING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue());
            ConvertBean.serviceModelUpdate(detail, AuthUtil.getSessionUser().getUserName());

            //根据扫描的条码查询条码是否存在
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
            DdlPreparedPack.getStringEqualPack(detail.getBarcode(), "productSn", packBean);
            DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
            MesProduceSn produceSn = produceSnRepository.getByProperty(packBean);

            produceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.SHIPPED.getValue());
            detail.setActualQty(1);
            ConvertBean.serviceModelUpdate(detail, AuthUtil.getSessionUser().getUserName());
            produceSnList.add(produceSn);

        }

        //当前发运单明细 需要报工的数据
        List<MesJisShipping> jisShippingList = jisShippingService.getJisShipping(orderManagementDetailList, org);

        produceSnRepository.saveAll(produceSnList);
        shippingOrderManagementRepository.save(orderManagement);
        shippingOrderManagementDetailRepository.saveAll(orderManagementDetailList);

        //保存当前发运单明细报工
        jisShippingRepository.saveAll(jisShippingList);
    }

}
