package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesWorkOrderService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description :
 * @Reference :
 * @Author :
 * @CreateDate : 2024-05-09 10:54
 * @Modify:
 **/

@Service
@Slf4j
public class MesWorkOrderService implements IMesWorkOrderService {

    @Autowired
    private MesWorkOrderRepository workOrderRepository;
    @Autowired
    private MesProductVersionRepository mesProductVersionRDao;
    @Autowired
    private MesBomRepository mesBomRDao;
    @Autowired
    private MesWorkCenterRepository mesWorkCenterRDao;
    @Autowired
    private MesPartRepository mesPartRDao;
    @Autowired
    private MesProductPlanRepository mesProductPlanRDao;
    @Autowired
    private MesProductOffLineRepository mesProductOffLineRDao;

    @Override
    public MesWorkOrder queryMesWorkOrder(MesWorkOrder workOrder) {

        DdlPackBean packBean = getDdlPackBean(workOrder);
        MesWorkOrder result = workOrderRepository.getByProperty(packBean);

        if (StringUtils.isEmpty(result)) {
            throw new ImppBusiException("查询工单不存在，请检查数据");
        }

        return result;
    }

    @Override
    public List<MesWorkOrder> queryMesWorkOrderList(MesWorkOrder workOrder) {

        DdlPackBean packBean = getDdlPackBean(workOrder);

        return workOrderRepository.findByHqlWhere(packBean);
    }

    @Override
    public ListPager<MesWorkOrder> queryMesWorkOrderListByPager(MesWorkOrder workOrder, Pager pager) {


        DdlPackBean packBean = getDdlPackBean(workOrder);

        pager = PagerHelper.getPager(pager, workOrderRepository.findByHqlWhereCount(packBean));

        List<MesWorkOrder> resultList = workOrderRepository.findByHqlWherePage(packBean, pager);

        return new ListPager<>(resultList, pager);
    }

    private DdlPackBean getDdlPackBean(MesWorkOrder workOrder) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(workOrder.getOrganizeCode());
        DdlPreparedPack.getStringLikerPack(workOrder.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringLikerPack(workOrder.getWorkCellCode(), "workCellCode", packBean);
        DdlPreparedPack.getStringLikerPack(workOrder.getWorkOrderNo(), "workOrderNo", packBean);
        DdlPreparedPack.getInPackList(workOrder.getStatusList(), "status", packBean);
        return packBean;
    }


    @Override
    public void doProductReport(List<MesProduceSn> mesProduceSnList) {
        //新增初始化
        String organizeCode = AuthUtil.getOrganizeCode();
        String userName = AuthUtil.getSessionUser().getUserName();

        Map<String, List<MesProduceSn>> mesWorkOrderMap = mesProduceSnList.stream().collect(Collectors.groupingBy(MesProduceSn::getPartNo));
        Map<String,MesPart> mesPartMap = new HashMap<>();
        MesPart mesPart = null;
        //查询工单状态
        Integer[] orderStatus =new Integer[]{MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue(),MesExtEnumUtil.ORDER_STATUS.process.getValue()};
        for (Map.Entry<String, List<MesProduceSn>> mesProduceSn : mesWorkOrderMap.entrySet()) {
            List<MesProduceSn> mesWorkOrderList = mesProduceSn.getValue();
            //获取物料信息
            if(mesPartMap.containsKey(mesProduceSn.getKey())){
                mesPart = mesPartMap.get(mesProduceSn.getKey());
            }else{
                //查询物料信息
                DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(mesProduceSn.getKey(), "partNo", ddlPackBean);
                mesPart = mesPartRDao.getByProperty(ddlPackBean);
                if(Objects.isNull(mesPart)){
                    throw new ImppBusiException(String.format("物料【%s】信息不存在", mesProduceSn.getKey()));
                }
                mesPartMap.put(mesProduceSn.getKey(),mesPart);
            }
            //根据物料获取已发布的工单
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
            DdlPreparedPack.getInPackArray(orderStatus, "workOrderStatus", ddlPackBean);
            MesWorkOrder oldMesWorkOrder = workOrderRepository.getByProperty(ddlPackBean);
            if(Objects.isNull(oldMesWorkOrder)){
                throw new ImppBusiException(String.format("物料【%s】状态未已发布的工单信息不存在", mesPart.getPartNo()));
            }
            //查询工作中心
            ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(oldMesWorkOrder.getWorkCenterCode(), "workCenterCode", ddlPackBean);
            MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(ddlPackBean);
            if (Objects.isNull(mesWorkCenter)) {
                throw new ImppBusiException(String.format("产线【%s】不存在", oldMesWorkOrder.getWorkCenterCode()));
            }
            //获取生产版本
            ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(mesWorkCenter.getErpWorkCenter(), "workCenterCode", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(oldMesWorkOrder.getProductVersion(), "productVersion", ddlPackBean);
            MesProductVersion mesProductVersion = mesProductVersionRDao.getByProperty(ddlPackBean);
            if (null == mesProductVersion) {
                throw new ImppBusiException(String.format("物料【%s】生产版本信息不存在", mesPart.getPartNo()));
            }
            //物料+生产版本获取bom信息
            ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesPart.getPartNo(), "partNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(mesProductVersion.getAlternativePartList(), "bomVersion", ddlPackBean);
            List<MesBom> mesBoms = mesBomRDao.findByHqlWhere(ddlPackBean);
            if (CollectionUtils.isEmpty(mesBoms)) {
                throw new ImppBusiException(String.format("物料【%s】生产版本【%s】对应bom信息不存在",
                        mesPart.getPartNo(), mesProductVersion.getProductVersion()));
            }
            oldMesWorkOrder.setNum(mesWorkOrderList.size());
            oldMesWorkOrder.setReportedQty(MathOperation.add(oldMesWorkOrder.getNum(), oldMesWorkOrder.getReportedQty()));
            //更新SAP计划完成数量
            saveMesProductPlan(oldMesWorkOrder, false, true);

            //更新工单状态
            double unCompleteQty = MathOperation.sub(oldMesWorkOrder.getQty(), oldMesWorkOrder.getReportedQty());
            oldMesWorkOrder.setUnCompleteQty(unCompleteQty > 0 ? unCompleteQty : 0);
            if (oldMesWorkOrder.getReportedQty() > oldMesWorkOrder.getQty()) {
                throw new ImppBusiException(String.format("工单报工数量【%s】大于工单数量【%s】,不允许报工",
                        oldMesWorkOrder.getReportedQty(), oldMesWorkOrder.getQty()));
            } else if (Objects.equals(oldMesWorkOrder.getReportedQty(), oldMesWorkOrder.getQty())) {
                oldMesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
            } else {
                oldMesWorkOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.process.getValue());
            }
            ConvertBean.serviceModelUpdate(oldMesWorkOrder,userName);
            workOrderRepository.update(oldMesWorkOrder);
            //保存数据
            List<MesProductOffLine> mesProductOffLineList = new ArrayList<>();
            MesProductOffLine newMesProductOffLine;
            for (MesProduceSn produceSn : mesWorkOrderList) {
                for (MesBom mesBom : mesBoms) {
                    newMesProductOffLine = new MesProductOffLine();
                    newMesProductOffLine.setReportPartNo(oldMesWorkOrder.getPartNo());
                    newMesProductOffLine.setReportPartNameRdd(oldMesWorkOrder.getPartNameRdd());
                    newMesProductOffLine.setItemPartNo(mesBom.getItemPartNo());
                    newMesProductOffLine.setItemPartNameRdd(mesBom.getItemPartName());
                    newMesProductOffLine.setItemQty(mesBom.getItemQty());
                    newMesProductOffLine.setAlort(mesProductVersion.getReceiveInventoryPoint());
                    newMesProductOffLine.setStgeLoc(mesProductVersion.getShipInventoryPoint());
                    newMesProductOffLine.setQty(1d);
                    newMesProductOffLine.setReportSn(produceSn.getProductSn());
                    newMesProductOffLine.setBomVersion(oldMesWorkOrder.getProductVersion());
                    newMesProductOffLine.setSerialNumber(produceSn.getProductSn());
                    newMesProductOffLine.setUnit(mesBom.getUnit());
                    newMesProductOffLine.setItemUnit(mesBom.getItemUnit());
                    newMesProductOffLine.setWorkOrderNo(oldMesWorkOrder.getWorkOrderNo());
                    newMesProductOffLine.setWorkCenterCode(oldMesWorkOrder.getWorkCenterCode());
                    newMesProductOffLine.setWorkCellCode(oldMesWorkOrder.getWorkCellCode());
                    newMesProductOffLine.setReportType(oldMesWorkOrder.getReportType());
                    newMesProductOffLine.setSapWorkCenter(oldMesWorkOrder.getErpWorkCenter());
                    newMesProductOffLine.setOrganizeCode(oldMesWorkOrder.getOrganizeCode());

                    ConvertBean.serviceModelInitialize(newMesProductOffLine, userName);
                    mesProductOffLineList.add(newMesProductOffLine);
                }
            }
            mesProductOffLineRDao.saveAll(mesProductOffLineList);
        }
    }

    private void saveMesProductPlan(MesWorkOrder bean, boolean isInsert, boolean isReport) {
        if (StringUtil.isEmpty(bean.getPlanOrderNo())) {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getPartNo(), "planPartNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(bean.getPlanStartTime(), "planStartDate", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(bean.getPlanEndTime(), "planEndDate", ddlPackBean);
            MesProductPlan mesProductPlan = mesProductPlanRDao.getByProperty(ddlPackBean);
            if (null == mesProductPlan) {
                MesProductPlan mesPlanOrder = new MesProductPlan();
                mesPlanOrder.setPlanOrderNo("");
                mesPlanOrder.setPlanQty(bean.getQty());
                mesPlanOrder.setPlanPartNo(bean.getPartNo());
                mesPlanOrder.setCompleteQty(0d);
                mesPlanOrder.setUncompleteQty(bean.getQty());
                mesPlanOrder.setUnit(bean.getUnit());
                mesPlanOrder.setPlanStartDate(bean.getPlanStartTime());
                mesPlanOrder.setPlanEndDate(bean.getPlanEndTime());
                mesPlanOrder.setPartMappingWorkCenterCode(bean.getErpWorkCenter());
                mesPlanOrder.setPlanOrganizeCode(bean.getOrganizeCode());
                mesPlanOrder.setOrganizeCode(bean.getOrganizeCode());
                ConvertBean.serviceModelInitialize(mesPlanOrder, bean.getCreateUser());
                mesProductPlanRDao.insert(mesPlanOrder);
            } else {
                if (isInsert) {
                    mesProductPlan.setPlanQty(MathOperation.add(mesProductPlan.getPlanQty(), bean.getQty()));
                } else {
                    //报工
                    if (isReport) {
                        mesProductPlan.setCompleteQty(MathOperation.add(mesProductPlan.getCompleteQty(), bean.getNum()));
                        //报工调整
                    } else {
                        mesProductPlan.setCompleteQty(MathOperation.sub(mesProductPlan.getCompleteQty(), bean.getNum()));
                    }
                }
                mesProductPlan.setUncompleteQty(MathOperation.sub(mesProductPlan.getPlanQty(), mesProductPlan.getCompleteQty()));
                ConvertBean.serviceModelUpdate(mesProductPlan, bean.getCreateUser());
                mesProductPlanRDao.update(mesProductPlan);
            }
        } else {
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(bean.getPlanOrderNo(), "planOrderNo", ddlPackBean);
            DdlPreparedPack.getStringEqualPack(bean.getPartNo(), "planPartNo", ddlPackBean);
            MesProductPlan mesProductPlan = mesProductPlanRDao.getByProperty(ddlPackBean);
            if (null == mesProductPlan) {
                throw new ImppBusiException(String.format("SAP计划单号【%s】物料【%s】,不存在", bean.getPlanOrderNo(), bean.getPartNo()));
            }
            if (!isInsert) {
                if (Objects.isNull(mesProductPlan.getCompleteQty())) {
                    mesProductPlan.setCompleteQty(0d);
                }
                //报工
                if (isReport) {
                    mesProductPlan.setCompleteQty(MathOperation.add(mesProductPlan.getCompleteQty(), bean.getNum()));
                    //报工调整
                } else {
                    mesProductPlan.setCompleteQty(MathOperation.sub(mesProductPlan.getCompleteQty(), bean.getNum()));
                }
                mesProductPlan.setUncompleteQty(MathOperation.sub(mesProductPlan.getPlanQty(), mesProductPlan.getCompleteQty()));
                ConvertBean.serviceModelUpdate(mesProductPlan, bean.getCreateUser());
                mesProductPlanRDao.update(mesProductPlan);
            }
        }
    }

}
