package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesEquipmentExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesProdOrgExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesCellEquipContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.actor.shipping.dispatch.IFsmRouteDataService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.mes.pcn.util.StationKvBeanUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.StationKvBean;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description : 获取生产过程上下文对象接口实现
 * @Author : wangjie
 **/
@Slf4j
@Service("mesProductionProcessContextStepService")
public class MesProductionProcessContextStepService extends BaseStepService implements IMesProductionProcessContextStepService {

    @Autowired
    private IMesProdOrgExtService prodOrgExtService;

    @Autowired
    private IMesEquipmentExtService equipmentExtService;

    @Autowired
    private IFsmRouteDataService fsmRouteDataService;

    private String getContextKey(StationRequestBean reqBean) { return getFsmBusikey(reqBean, MesPcnExtConstWords.PRODUCTION_PROCESS_CONTEXT); }

    @Override
    public Boolean doCacheProductionProcessContext(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext) {
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesProductionProcessContext.class.getSimpleName(), JSONObject.toJSONString(productionProcessContext));
    }

    @Override
    public MesProductionProcessContext getProductionProcessContext(StationRequestBean reqBean) {

        //获取缓存的生产过程上下文对象
        MesProductionProcessContext productionProcessContext = (MesProductionProcessContext) getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesProductionProcessContext.class.getSimpleName(), MesProductionProcessContext.class);

        if (null == productionProcessContext) productionProcessContext = new MesProductionProcessContext(reqBean.getOrganizeCode());

        //生产过程上下文对象结果初始化
        productionProcessContext.success();

        //生产过程上下文对象赋值生产线信息
        if (StringUtils.isEmpty(productionProcessContext.getWorkCenterJson())) productionProcessContext.workCenterJson(prodOrgExtService.getWorkCenterDb(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode()));

        //生产过程上下文对象赋值工位信息
        if (StringUtils.isEmpty(productionProcessContext.getWorkCellJson())) productionProcessContext.workCellJson(prodOrgExtService.getWorkCellDb(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode(), reqBean.getWorkCellCode()));

        //生产过程上下文对象赋值工序工艺信息
        if (StringUtils.isEmpty(productionProcessContext.getProcessCode()) || !productionProcessContext.getProcessCode().equals(reqBean.getProcessCode())) processContext(reqBean, productionProcessContext);

        //验证组织模型有效性
        return checkBaseData(reqBean, productionProcessContext);

    }

    @Override
    public MesProductionProcessContext processContext(StationRequestBean reqBean) {
        return processContext(reqBean, getProductionProcessContext(reqBean));
    }

    @Override
    public MesProductionProcessContext processContext(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext) {

        if (!productionProcessContext.getSuccess()) return productionProcessContext;

        if (!StringUtils.isEmpty(productionProcessContext.getProcessCode()) && productionProcessContext.getProcessCode().equals(reqBean.getProcessCode())) return productionProcessContext;

        MesProcess process = fsmRouteDataService.doCachedFsmProcessData(reqBean, reqBean.getProcessCode());

        MesProcessCraftCfg processCraftCfg = fsmRouteDataService.getProcessCraftCfg(reqBean.getOrganizeCode(), reqBean.getProcessCode());

        //生产过程上下文对象赋值当前工艺工序,未配置默认赋值工序代码,不强验证
        productionProcessContext.processCode(reqBean.getProcessCode(), null != process ? process.getProcessName() : reqBean.getProcessCode());

        return productionProcessContext.craftCode(null != processCraftCfg ? processCraftCfg.getCraftCode() : reqBean.getProcessCode(), (null != processCraftCfg) ? productionProcessContext.getProcessCode() : productionProcessContext.getProcessName());

    }

    @Override
    public MesProductionProcessContext checkBaseData(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext) {

        if (StringUtils.isEmpty(productionProcessContext.getWorkCenterJson())) return productionProcessContext.message(String.format("请检查组织模型生产线[%s]信息的有效性!", reqBean.getWorkCenterCode()));

        if (StringUtils.isEmpty(productionProcessContext.getWorkCellJson())) return productionProcessContext.message(String.format("请检查组织模型生产线[%s]工位[%s]信息的有效性!", reqBean.getWorkCenterCode(), reqBean.getWorkCellCode()));

        if (StringUtils.isEmpty(productionProcessContext.getProcessCode())) return productionProcessContext.message(String.format("请检查工序信息,生产线[%s]工位[%s]对应工序代码[%s]的有效性!", reqBean.getWorkCenterCode(), reqBean.getWorkCellCode(), reqBean.getProcessCode()));

        return productionProcessContext;

    }

    @Override
    public MesProductionProcessContext getCellEquipmentList(StationRequestBean reqBean) {
        return getCellEquipmentList(reqBean, getProductionProcessContext(reqBean));
    }

    @Override
    public MesProductionProcessContext getCellEquipmentList(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext) {

        if (!productionProcessContext.getSuccess()) return productionProcessContext;

        //生产过程上下文对象赋值设备信息
        if (StringUtils.isEmpty(productionProcessContext.getCellEquipListJson())) productionProcessContext.cellEquipListJson(equipmentExtService.getCellEquipmentList(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode(), reqBean.getWorkCellCode()));

        return productionProcessContext;

    }

    @Override
    public MesProductionProcessContext getCurCellEquipment(StationRequestBean reqBean) {
        return getCurCellEquipment(reqBean, getCellEquipmentList(reqBean));
    }

    @Override
    public MesProductionProcessContext getCurCellEquipment(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext) {

        if (!productionProcessContext.getSuccess()) return productionProcessContext;

        return getCurCellEquipment(reqBean, productionProcessContext, getStepParams(reqBean));

    }

    @Override
    public MesProductionProcessContext getCurCellEquipment(StationRequestBean reqBean, Optional<Map<String, MesProdRouteOptParam>> stepParamMap) {
        return getCurCellEquipment(reqBean, getCellEquipmentList(reqBean), stepParamMap);
    }

    @Override
    public MesProductionProcessContext getCurCellEquipment(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext, Optional<Map<String, MesProdRouteOptParam>> stepParamMap) {

        if (!productionProcessContext.getSuccess()) return productionProcessContext;

        if (StringUtils.isEmpty(productionProcessContext.getCellEquipListJson())) getCellEquipmentList(reqBean, productionProcessContext);

        //获取当前工步的设备参数
        String equipmentCode = (null != stepParamMap && stepParamMap.isPresent() && stepParamMap.get().containsKey(MesPcnExtConstWords.EQUIPMENT_CODE_UC)) ? stepParamMap.get().get(MesPcnExtConstWords.EQUIPMENT_CODE_UC).getParamValue() : null;

        //判断生产过程上下文对象是否存在当前设备信息
        if (!StringUtils.isEmpty(productionProcessContext.getCurCellEquipJson()) && (StringUtils.isEmpty(equipmentCode) || equipmentCode.equals(productionProcessContext.getCurCellEquip().getEquipmentCode()))) return productionProcessContext;

        //从工位设备集合信息中获取当前设备信息
        MesCellEquipContext cellEquipmentContext = productionProcessContext.filterCurCellEquip(equipmentCode);

        //生产过程上下文对象赋值当前设备代码
        if (null != cellEquipmentContext) return productionProcessContext.curCellEquipJson(cellEquipmentContext);

        return StringUtils.isEmpty(equipmentCode) ? productionProcessContext.message(String.format("请检查组织模型,生产线[%s]工位[%s]未维护设备信息!", reqBean.getWorkCenterCode(), reqBean.getWorkCellCode())) :
                productionProcessContext.message(String.format("请检查组织模型,生产线[%s]工位[%s]未维护工步参数[%s]配置的设备[%s]!", reqBean.getWorkCenterCode(), reqBean.getWorkCellCode(), MesPcnExtConstWords.EQUIPMENT_CODE_UC, equipmentCode));

    }

    @Override
    public MesProductionProcessContext getEquipmentVariableCfgList(StationRequestBean reqBean) {
        return getEquipmentVariableCfgList(getCurCellEquipment(reqBean));
    }

    @Override
    public MesProductionProcessContext getEquipmentVariableCfgList(MesProductionProcessContext productionProcessContext) {

        if (!productionProcessContext.getSuccess()) return productionProcessContext;

        //判断生产过程上下文对象是否存在数据变量接口逻辑信息
        if (!CollectionUtils.isEmpty(productionProcessContext.getEquipVariableCfgListByVct())) return productionProcessContext;

        MesCellEquipContext cellEquipmentContext = productionProcessContext.getCurCellEquip();

        if (null == cellEquipmentContext || !productionProcessContext.getSuccess()) return productionProcessContext;

        List<MesEquipmentVariableCfg> equipmentVariableCfgList = equipmentExtService.getEquipmentVariableCfgList(productionProcessContext.getOrganizeCode(), cellEquipmentContext.getVariableCfgType());

        //生产过程上下文对象赋值设备数据变量接口逻辑集合信息
        if (!CollectionUtils.isEmpty(equipmentVariableCfgList)) productionProcessContext.equipVariableCfgJson(equipmentVariableCfgList);

        return productionProcessContext;

    }

    @Override
    public List<MesEquipmentVariableCfg> collectEquipmentVariableCfgList(List<MesEquipmentVariableCfg> equipmentVariableCfgList, String... variableCategory) {
        List<String> variableCategoryList = null == variableCategory ? null : Arrays.asList(variableCategory);
        return (CollectionUtils.isEmpty(equipmentVariableCfgList) || CollectionUtils.isEmpty(variableCategoryList)) ? null : equipmentVariableCfgList.stream().filter(o -> (null != o && variableCategoryList.contains(o.getVariableCategory()))).collect(Collectors.toList());
    }

    @Override
    public MesProductionProcessContext checkNecessaryEquipmentVariableCfg(MesProductionProcessContext productionProcessContext, MesCellEquipContext cellEquipContext, List<MesEquipmentVariableCfg> equipmentVariableCfgList, String... variableCategory) {
        List<String> variableCategoryList = null == variableCategory ? null : Arrays.asList(variableCategory);
        Optional<MesEquipmentVariableCfg> optional = (CollectionUtils.isEmpty(equipmentVariableCfgList) || CollectionUtils.isEmpty(variableCategoryList)) ? null : equipmentVariableCfgList.stream().filter(o -> (null != o && variableCategoryList.contains(o.getVariableCategory()))).findFirst();
        return (null != optional && optional.isPresent()) ? productionProcessContext : productionProcessContext.message(String.format("请检查设备数据变量接口逻辑信息,设备[%s]未配置变量配置类型[%s]变量类别%s的数据变量接口逻辑信息!", cellEquipContext.getEquipmentCode(), cellEquipContext.getVariableCfgType(), variableCategoryList.toString()));
    }

    @Override
    public List<String> collectCategoryLevelTwoList(List<MesEquipmentVariableCfg> equipmentVariableCfgList) {
        return CollectionUtils.isEmpty(equipmentVariableCfgList) ? null :
                (equipmentVariableCfgList.stream().filter(o -> null != o).map(MesEquipmentVariableCfg::getCategoryLevelTwo).collect(Collectors.toList())).stream().filter(o -> StringUtils.isEmpty(o)).distinct().collect(Collectors.toList());
    }

    @Override
    public MesProductionProcessContext getEquipmentVariableList(StationRequestBean reqBean, Integer variableType) {
        return getEquipmentVariableList(getEquipmentVariableCfgList(reqBean), variableType);
    }

    @Override
    public MesProductionProcessContext getEquipmentVariableList(MesProductionProcessContext productionProcessContext, Integer variableType) {

        if (!productionProcessContext.getSuccess() || StringUtils.isEmpty(variableType)) return productionProcessContext;

        MesCellEquipContext cellEquipmentContext = productionProcessContext.getCurCellEquip();

        //判断生产过程上下文对象是否存在数据变量信息
        if (!CollectionUtils.isEmpty(productionProcessContext.getEquipVariableListByEquVt(cellEquipmentContext.getEquipmentCode(), variableType))) return productionProcessContext;

        List<MesEquipmentVariable> equipmentVariableDbList = equipmentExtService.getEquipmentVariableList(productionProcessContext.getOrganizeCode(), cellEquipmentContext.getEquipmentCode(), variableType);

        //生产过程上下文对象赋值设备数据变量信息
        if (null != equipmentVariableDbList) productionProcessContext.equipVariableJson(equipmentVariableDbList);

        return productionProcessContext;

    }

    @Override
    public List<MesEquipmentVariable> findEquipmentVariableList(MesProductionProcessContext productionProcessContext, Integer variableType) {

        if (!productionProcessContext.getSuccess() || StringUtils.isEmpty(variableType)) return null;

        MesCellEquipContext cellEquipmentContext = productionProcessContext.getCurCellEquip();

        //判断生产过程上下文对象是否存在数据变量信息
        List<MesEquipmentVariable> equipmentVariableDbList = productionProcessContext.getEquipVariableListByEquVt(cellEquipmentContext.getEquipmentCode(), variableType);
        if (!CollectionUtils.isEmpty(equipmentVariableDbList)) return equipmentVariableDbList;

        equipmentVariableDbList = equipmentExtService.getEquipmentVariableList(productionProcessContext.getOrganizeCode(), cellEquipmentContext.getEquipmentCode(), variableType);

        //生产过程上下文对象赋值设备数据变量信息
        if (null != equipmentVariableDbList) productionProcessContext.equipVariableJson(equipmentVariableDbList);

        return equipmentVariableDbList;

    }

    @Override
    public List<MesEquipmentVariable> findEquipmentVariableList(MesProductionProcessContext productionProcessContext, Integer variableType, List<String> categoryLevelTwoList) {
        List<MesEquipmentVariable> equipmentVariableList = findEquipmentVariableList(productionProcessContext, variableType);
        return findEquipmentVariableList(equipmentVariableList, variableType, categoryLevelTwoList);
    }

    @Override
    public List<MesEquipmentVariable> findEquipmentVariableList(List<MesEquipmentVariable> equipmentVariableList, Integer variableType, List<String> categoryLevelTwoList) {
        return (CollectionUtils.isEmpty(equipmentVariableList) || CollectionUtils.isEmpty(categoryLevelTwoList)) ? null :
                equipmentVariableList.stream().filter(o -> (null != o && categoryLevelTwoList.contains(o.getCategoryLevelTwo()) && o.getVariableType().compareTo(variableType) == 0)).collect(Collectors.toList());
    }

    @Override
    public void sendEquipQualityMessage(StationRequestBean reqBean, MesCellEquipContext cellEquipContext) {
        StationResultBean resultBean = new StationResultBean();
        resultBean.setResultObj(MesExtEnumUtil.EQUIP_LOG_QUALITY.valueOfColor(cellEquipContext.getQuality()));
        resultBean.setBusiType(MesPcnEnumUtil.STATION_BUSI_TYPE.QUALITY_MODULE.getValue());
        resultBean.setDataType(MesPcnEnumUtil.STATION_DATA_TYPE.COLOR.getValue());
        this.sendMessage(reqBean, resultBean);
    }



    @Override
    public Boolean doCacheModuleContext(StationRequestBean reqBean, List<List<StationKvBean>> dataList) {
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnEnumUtil.STATION_BUSI_TYPE.MODULE_CONTENT.getValue(), JSONObject.toJSONString(dataList));
    }

    @Override
    public List<List<StationKvBean>> getCachedModuleContext(StationRequestBean reqBean) {
        String moduleContent = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnEnumUtil.STATION_BUSI_TYPE.MODULE_CONTENT.getValue());
        return StringUtils.isEmpty(moduleContent) ? null : JSONObject.parseObject(moduleContent, new TypeReference<List<List<StationKvBean>>>() {});
    }

    @Override
    public Boolean doCacheLastScanContext(StationRequestBean reqBean, List<StationKvBean> dataList) {
        return saveFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.LAST_SCAN_INFO, JSONObject.toJSONString(dataList));
    }

    @Override
    public List<StationKvBean> getCachedLastScanContext(StationRequestBean reqBean) {
        String lastScanContent = getFsmBusiData(reqBean.getOrganizeCode(), getContextKey(reqBean), MesPcnExtConstWords.LAST_SCAN_INFO);
        return StringUtils.isEmpty(lastScanContent) ? new ArrayList<>() : JSONObject.parseArray(lastScanContent, StationKvBean.class);
    }

    @Override
    public void doSendStepContextMessage(StationRequestBean reqBean) {
        sendStepContextMessage(reqBean, getCachedLastScanContext(reqBean));
    }

    @Override
    public void doSendStepContextMessage(StationRequestBean reqBean, String scanInfo, MesExtEnumUtil.CELL_MESSAGE_SOURCE source) {

        List<StationKvBean> lastScanContext = getCachedLastScanContext(reqBean);

        StationKvBeanUtil.addStationKvBeanLimix100ThenHalf(lastScanContext, new StationKvBean().name(source.getDescription()).value(scanInfo));

        doCacheLastScanContext(reqBean, lastScanContext);

        sendStepContextMessage(reqBean, lastScanContext);

    }

    @Override
    public void sendStepContextMessage(StationRequestBean reqBean, List<StationKvBean> lastScanContext) {
        StationResultBean resultBean = new StationResultBean();
        resultBean.setResultList(StationKvBeanUtil.addStationKvBeanList(new ArrayList<>(),
                lastScanContext.size() > 1 ? lastScanContext.get(1).name("上次" + lastScanContext.get(1).getName()).key(MesPcnExtConstWords.LAST_SCAN_INFO) : null,
                lastScanContext.size() > 0 ? lastScanContext.get(0).name("当前" + lastScanContext.get(0).getName()).key(MesPcnExtConstWords.SCAN_INFO) : null));
        resultBean.setBusiType(MesPcnEnumUtil.STATION_BUSI_TYPE.STEP_CONTENT.getValue());
        resultBean.setDataType(MesPcnEnumUtil.STATION_DATA_TYPE.KEY_VALUE.getValue());
        this.sendMessage(reqBean, resultBean);
    }

}
